#include <kernel.h>
#include <t_syslog.h>
#include <stdio.h>
#include <string.h>
#include <t_stdlib.h>
#include <stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "monitor.h"
#include "device.h"
#include "qspi.h"

#define MIN(a, b) ((a) > (b) ? (b) : (a))

/*
 *  QSPIޥֹ
 */
static int_t qspi_dumpdata(int argc, char **argv);
static int_t qspi_filldata(int argc, char **argv);
static int_t qspi_erasedata(int argc, char **argv);

/*
 *  QSPIޥɥơ֥
 */
static const COMMAND_INFO qspi_command_info[] = {
	{"DUMP",    qspi_dumpdata},
	{"FILL",    qspi_filldata},
	{"ERASE",   qspi_erasedata}
};

#define NUM_QSPI_CMD (sizeof(qspi_command_info)/sizeof(COMMAND_INFO))

static const char qspi_name[] = "QSPI";
static const char qspi_help[] =
"  QSPI    DUMP   addr size  \n"
"          FILL   addr size value \n"
"          ERASE  addr size  \n";

static COMMAND_LINK qspi_command_link = {
	NULL,
	NUM_QSPI_CMD,
	qspi_name,
	NULL,
	qspi_help,
	&qspi_command_info[0]
};

static QSPI_Handle_t *hqspi;

/*
 *  QSPIޥؿ
 */
void qspi_info_init(intptr_t exinf)
{
	setup_command(&qspi_command_link);
	hqspi = (QSPI_Handle_t *)exinf;
}

static unsigned int get_hex(char *s)
{
	unsigned int val = 0, c;

	while((c = *s++) != 0){
		val *= 16;
		if(c >= '0' && c <= '9')
			val += c - '0';
		else if(c >= 'A' && c <= 'Z')
			val += c - 'A' + 10;
		else if(c >= 'a' && c <= 'z')
			val += c - 'a' + 10;
	}
	return val;
}

/*
 *  QSPIɤ߽Ф
 */
static int_t qspi_dumpdata(int argc, char **argv)
{
	uint32_t addr;
	uint8_t  *buf;
	uint32_t io_block_size;
	int      size, len, i;
	ER       ercd = E_OK;

	if(hqspi == NULL)
		return -1;
	if(argc == 2)
		size = 512;
	else if(argc > 2)
		size = get_hex(argv[2]);
	else
		return -1;
	addr = get_hex(argv[1]);
	if((addr & 3) != 0)
		return -1;
	io_block_size = hqspi->Init.erase_sizes[0];
	buf = malloc(io_block_size);
	if(buf == NULL)
		return -1;

	while(size > 0){
		len = MIN(size, io_block_size);
		ercd = qspi_read(hqspi, (void*)buf, (uint32_t)addr, len);
		if(ercd != E_OK)
			printf("qspi read error[%08x]\n", addr);
		for(i = 0 ; i < len ; i++){
			if((i % 16) == 0)
				printf("\n %08x", addr + i);
			printf(" %02x", buf[i]);
		}
		addr += len;
		size -= len;
	}
	printf("\n");
	free(buf);
	return 0;
}

/*
 *  QSPI FILL񤭹
 */
static int_t qspi_filldata(int argc, char **argv)
{
	uint32_t size = 0, io_block_size, addr;
	uint8_t  *buf, value;
	int  len, i;
	ER   ercd;

	if(argc < 4 || hqspi == NULL)
		return -1;
	addr = get_hex(argv[1]);
	io_block_size = hqspi->Init.erase_sizes[0];
	buf = malloc(io_block_size);
	if(buf == NULL){
		printf("allocation error[%08x] !\n", io_block_size);
		return -1;
	}
	size  = get_hex(argv[2]);
	value = get_hex(argv[3]);
	if(size > hqspi->Init.device_size)
		return -1;

	printf("set addr[%08x] size[%08x] value[%02x]\n", addr, size, value);
	for(i = 0 ; i < size ; i += io_block_size){
		ercd = qspi_erase_sector(hqspi, addr + i);
		if(ercd != E_OK){
			printf("erase error[%08x] !\n", addr+i);
			return -1;
		}
		memset(buf, value, io_block_size);
		len = MIN(io_block_size, size-i);
		len = (len + 3) & ~3;
		ercd = qspi_write(hqspi, addr + i, buf, len);
		if(ercd != E_OK){
			printf("write error[%08x] !\n", i);
			free(buf);
			return -1;
		}
		printf(".");
	}
	free(buf);
	printf("\nfinished !\n");
	return 0;
}

/*
 *  QSPI ERASE
 */
static int_t qspi_erasedata(int argc, char **argv)
{
	uint32_t size = 0, io_block_size, addr;
	uint32_t i;
	ER       ercd;

	if(argc < 3 || hqspi == NULL)
		return -1;
	addr = get_hex(argv[1]);
	io_block_size = hqspi->Init.erase_sizes[0];
	if(argc >= 3)
		size = get_hex(argv[2]);
	else
		size = io_block_size;
	addr &= ~(io_block_size-1);

	for(i = 0 ; i < size ; i += io_block_size){
		printf("erase [%08x]\n", addr + i);
		ercd = qspi_erase_sector(hqspi, addr + i);
		if(ercd != E_OK){
			printf("erase error[%08x] !\n", addr+i);
			return -1;
		}
	}
	printf("finished !\n");
	return 0;
}

