/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2020 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: i2c.c 698 2020-11-19 15:56:26Z roi $
 */
/*
 *
 *  I2Cドライバ関数群
 *
 */
#include "kernel.h"
#include <t_syslog.h>
#include <t_stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sil.h>
#include <target_syssvc.h>
#include "device.h"
#include "i2c.h"

#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_modw_mem(a, b, c)	sil_wrw_mem((a), (sil_rew_mem(a) & (~b)) | (c))

/*
 *  I2CポートIDから管理ブロックを取り出すためのマクロ
 */
#define INDEX_I2C(i2cid)        ((uint_t)((i2cid) - 1))

#define i2c_start(h)            i2c_trans_bit((h), (1), (0), (0))
#define i2c_stop(h)             i2c_trans_bit((h), (0), (1), (1))

#define SCL_DELAY               sil_dly_nse

#define BIT_WIDTH               8
#define GPIO_MODE               0x00000003

/*
 *  I2Cポート設定テーブル
 */
static const I2C_PortControlBlock i2c_pcb[NUM_I2CPORT] = {
  {	TADR_GPIO_BASE, 13, 12, GPIO_NOPULL, GPIO_NOIOF }
};

static I2C_Handle_t I2CHandle[NUM_I2CPORT];


/*
 *  GPIO設定関数
 */
static void
pinMode(I2C_Handle_t *hi2c, uint8_t no, uint32_t otype)
{
	GPIO_Init_t GPIO_Init_Data;

	GPIO_Init_Data.mode  = GPIO_MODE_OUTPUT;
	GPIO_Init_Data.pull  = hi2c->gpiopull;
	GPIO_Init_Data.otype = otype;
	GPIO_Init_Data.oxor  = GPIO_OUTPUT_NORMAL;
	GPIO_Init_Data.iof   = hi2c->gpioiof1;
	gpio_setup(hi2c->base, &GPIO_Init_Data, no);
	sil_orw_mem((uint32_t *)(hi2c->base+TOFF_GPIO_OUTPUT_VAL), (1 << no));
}

/*
 *  GPIO-SDAディレクション変更
 */
static void
i2c_change_direction(I2C_Handle_t *hi2c, uint32_t oe)
{
	uint32_t base = hi2c->base;
	uint32_t pin  = hi2c->sdapin;
	uint32_t tmp;

	if(hi2c->sdaoe == oe)
		return;
	if((oe & GPIO_MODE) == GPIO_MODE_INPUT){
		sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_EN), 1 << pin);
		sil_andw_mem((uint32_t *)(base+TOFF_GPIO_PULLUP_EN), 1 << pin);
		sil_orw_mem((uint32_t *)(base+TOFF_GPIO_INPUT_EN), 1 << pin);
	}
	else{
		sil_andw_mem((uint32_t *)(base+TOFF_GPIO_INPUT_EN), 1 << pin);
		if(hi2c->gpiopull == GPIO_PULLUP)
			sil_orw_mem((uint32_t *)(base+TOFF_GPIO_PULLUP_EN), 1 << pin);
		else
			sil_andw_mem((uint32_t *)(base+TOFF_GPIO_PULLUP_EN), 1 << pin);
		sil_orw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_EN), 1 << pin);
	}
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_EN));
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_INPUT_EN));
	(void)(tmp);
	hi2c->sdaoe = oe;
}

/*
 *  ビット転送
 */
static void
i2c_trans_bit(I2C_Handle_t *hi2c, uint8_t data1, uint8_t data2, uint8_t clk)
{
	uint32_t base = hi2c->base;
	uint32_t tmp;

	SCL_DELAY(hi2c->clkCount);
	i2c_change_direction(hi2c, GPIO_MODE_OUTPUT);

	/* SDAT=D1送信 */
	sil_modw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sdapin, (data1 & 1) << hi2c->sdapin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	sil_orw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);

	/* SDAT=D2..SCLK=0 */
	sil_modw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sdapin, (data2 & 1) << hi2c->sdapin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	if(clk == 0)
		sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	(void)(tmp);
}

/*
 *  ビット送信
 */
static void
i2c_send_bit(I2C_Handle_t *hi2c, uint8_t data)
{
	uint32_t base = hi2c->base;
	uint32_t tmp;

	/* SCLK= 0 */
	sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	i2c_change_direction(hi2c, GPIO_MODE_OUTPUT);

	/* SDAT=送信 */
	sil_modw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sdapin, (data & 1) << hi2c->sdapin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);

	/* SDAT= 0..SCLK=0 */
	sil_orw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	(void)(tmp);
}

/*
 *  ビット受信
 */
static int
i2c_recv_bit(I2C_Handle_t *hi2c)
{
	uint32_t base = hi2c->base;
	uint32_t tmp;
	int data;
	/* SCLK= 0 */
	sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	/* SCLK=1 */
	sil_orw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), (1 << hi2c->sclpin));
	sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), (1 << hi2c->sdapin));
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	/* SDAT入力 */
	i2c_change_direction(hi2c, GPIO_MODE_INPUT);
	SCL_DELAY(hi2c->clkCount);
	/* SDAT=受信 */
	data = (sil_rew_mem((uint32_t *)(base+TOFF_GPIO_INPUT_VAL)) >> hi2c->sdapin) & 1;
	syslog_2(LOG_DEBUG, "i2c_recv_bit sda(%d) data[%08x] !", hi2c->sdapin, sil_rew_mem((uint32_t *)(base+TOFF_GPIO_INPUT_VAL)));
	SCL_DELAY(hi2c->clkCount);
	sil_andw_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL), 1 << hi2c->sclpin);
	tmp = sil_rew_mem((uint32_t *)(base+TOFF_GPIO_OUTPUT_VAL));
	SCL_DELAY(hi2c->clkCount);
	(void)(tmp);
	return data;
}

/*
 *  バイト送信
 */
static int
i2c_send_byte(I2C_Handle_t *hi2c, uint8_t data)
{
	int     i;

	for(i = 0 ; i < BIT_WIDTH ; i++){
		i2c_send_bit(hi2c, (data >> (7-i)) & 1);
	}
	return i2c_recv_bit(hi2c) ^ 1;
}

/*
 *  バイト受信
 */
static uint8_t
i2c_recv_byte(I2C_Handle_t *hi2c)
{
	uint8_t data = 0;
	int     i;

	for(i = 0 ; i < BIT_WIDTH ; i++){
		i2c_change_direction(hi2c, GPIO_MODE_INPUT);
		SCL_DELAY(hi2c->clkCount * 2);
		data <<= 1;
		data |= i2c_recv_bit(hi2c) & 1;
	}
	i2c_send_bit(hi2c, 0);
	return data;
}


/*
 *  I2C初期設定
 *  parameter1  port: FPGASPIポート番号
 *  return I2Cハンドラへのポインタ、NULLでエラー
 */
I2C_Handle_t *
i2c_init(ID portid, I2C_Init_t *ii2c)
{
	I2C_Handle_t * hi2c;
	const I2C_PortControlBlock *ppcb;
	uint32_t no;

	if(portid > NUM_I2CPORT || portid == 0)
        return NULL;
	if(ii2c == NULL)
		return NULL;
	no = INDEX_I2C(portid);
	hi2c = &I2CHandle[no];
	if(hi2c->base != 0 || ii2c->ClockSpeed < 1000)
		return NULL;

	ppcb = &i2c_pcb[no];
	memset(hi2c, 0, sizeof(I2C_Handle_t));
	memcpy(&hi2c->Init, ii2c, sizeof(I2C_Init_t));
    hi2c->base = ppcb->base;
	hi2c->portid = portid;
	hi2c->clkCount  = 250*1000*1000 / ii2c->ClockSpeed;
	hi2c->sclpin    = ppcb->sclpin;
	hi2c->sdapin    = ppcb->sdapin;
	hi2c->gpiopull  = ppcb->gpiopull;
	hi2c->gpioiof1  = ppcb->gpioiof1;
	hi2c->sdaoe     = GPIO_MODE_OUTPUT;
	hi2c->ErrorCode = I2C_ERROR_NONE;
	pinMode(hi2c, hi2c->sclpin, GPIO_OTYPE_OD);
	pinMode(hi2c, hi2c->sdapin, GPIO_OTYPE_OD);
	return hi2c;
}

/*
 *  I2C終了設定
 *  parameter1  hi2c: I2Cハンドラへのポインタ
 *  return ERコード
 */
ER
fpgai2c_deinit(I2C_Handle_t * hi2c)
{
	if(hi2c == NULL)
		return E_PAR;
	if(hi2c->portid == 0 || hi2c->portid > NUM_I2CPORT)
		return E_PAR;
	hi2c->base   = 0;
    return E_OK;
}


/*
 *  I2Cデータリード
 *  parameter1 hi2c:       I2Cハンドラへのポインタ
 *  parameter2 DevAddress: スレーブアドレス
 *  parameter3 MemAddress: メモリアドレス
 *  parameter4 MemAddSize: メモリアドレスサイズ
 *  parameter5 pData:      読み出しバッファへのポインタ
 *  parameter6 Size:       読み出しサイズ
 *  return                 正常終了ならE_OK
 */
ER
i2c_memread(I2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, void * data, uint32_t Size)
{
	uint8_t *bdata = data;
	int     i;

	if(data == NULL || Size == 0 || hi2c == NULL)
		return E_PAR;

	/*
	 *  書き込みロック
	 */
	if(hi2c->Init.semlock != 0)
		wai_sem(hi2c->Init.semlock);

	/*
	 *  アドレス設定があれば送信
	 */
	if(addrsize > 0){
		/*
		 *  スレーブアドレス設定
	 	*/
		i2c_start(hi2c);
		if(i2c_send_byte(hi2c, devaddr & ~1) == 0){
			i2c_stop(hi2c);
			return E_OBJ;
		}
		if(addrsize == 1){
			if(i2c_send_byte(hi2c, addr) == 0){
				i2c_stop(hi2c);
				return E_OBJ;
			}
		}
		else if(addrsize > 1){
			if(i2c_send_byte(hi2c, addr>>8) == 0){
				i2c_stop(hi2c);
				return E_OBJ;
			}
			if(i2c_send_byte(hi2c, addr & 0xff) == 0){
				i2c_stop(hi2c);
				return E_OBJ;
			}
		}
		i2c_change_direction(hi2c, GPIO_MODE_OUTPUT);
		SCL_DELAY(500);
	}

	/*
	 *  受信設定と受信終了待ち
	 */
	i2c_start(hi2c);
	if(i2c_send_byte(hi2c, devaddr | 1) == 0){
		i2c_stop(hi2c);
		return E_OBJ;
	}
	for(i = 0 ; i < Size; i++){
		*bdata++ = i2c_recv_byte(hi2c);
	}
	i2c_stop(hi2c);

	/*
	 *  書き込みロック解除
	 */
	if(hi2c->Init.semlock != 0)
		sig_sem(hi2c->Init.semlock);
	return E_OK;
}

/*
 *  I2Cデータライト
 *  parameter1 hi2c:       I2Cハンドラへのポインタ
 *  parameter2 DevAddress: スレーブアドレス
 *  parameter3 MemAddress: メモリアドレス
 *  parameter4 MemAddSize: メモリアドレスサイズ
 *  parameter5 pData:      書込みバッファへのポインタ
 *  parameter6 Size:       書込みサイズ
 *  return                 正常終了ならE_OK
 */
ER
i2c_memwrite(I2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, const void * data, uint32_t Size)
{
	const uint8_t *bdata = data;
	ER    ercd = E_OK;
	int   i;

	if(data == NULL || Size == 0 || hi2c == NULL)
		return E_PAR;

	/*
	 *  書き込みロック
	 */
	if(hi2c->Init.semlock != 0)
		wai_sem(hi2c->Init.semlock);

	/*
	 *  スレーブアドレス設定
	 */
	i2c_start(hi2c);
	if(i2c_send_byte(hi2c, devaddr & ~1) == 0){
		i2c_stop(hi2c);
		return E_OBJ;
	}

	/*
	 *  アドレス設定があれば送信
	 */
	if(addrsize == 1){
		if(i2c_send_byte(hi2c, addr) == 0){
			i2c_stop(hi2c);
			return E_OBJ;
		}
		SCL_DELAY(250);
	}
	else if(addrsize > 1){
		if(i2c_send_byte(hi2c, addr>>8) == 0){
			i2c_stop(hi2c);
			return E_OBJ;
		}
		if(i2c_send_byte(hi2c, addr & 0xff) == 0){
			i2c_stop(hi2c);
			return E_OBJ;
		}
		SCL_DELAY(250);
	}

	/*
	 *  送信設定と送信終了待ち
	 */
	for(i = 0 ; i < Size ; i++){
		if(i2c_send_byte(hi2c, *bdata++) == 0){
			ercd = E_OBJ;
			break;
		}
	}
	i2c_stop(hi2c);

	/*
	 *  書き込みロック解除
	 */
	if(hi2c->Init.semlock != 0)
		sig_sem(hi2c->Init.semlock);
	return ercd;
}

/*
 *  I2Cレディチェック
 *  parameter1 hi2c:       I2Cハンドラへのポインタ
 *  parameter2 DevAddress: スレーブアドレス
 *  parameter3 Trials:     トライアス値
 *  parameter4 Timeout:    タイムアウト値
 *  return                 レディならE_OK
 */
ER
i2c_ready(I2C_Handle_t *hi2c, uint16_t DevAddress, uint32_t Trials, uint32_t Timeout)
{
	volatile uint32_t I2C_Trials = 0;
	ER ercd = E_TMOUT;

	/*
	 *  書き込みロック
	 */
	if(hi2c->Init.semlock != 0)
		wai_sem(hi2c->Init.semlock);

	do{
		i2c_start(hi2c);
		if(i2c_send_byte(hi2c, DevAddress & ~1) != 0){
			i2c_stop(hi2c);
			ercd = E_OK;
			break;
		}
		else{
			i2c_stop(hi2c);
			dly_tsk(50);
		}
	}while(I2C_Trials++ < Trials);

	/*
	 *  ロック解除
	 */
	if(hi2c->Init.semlock != 0)
		sig_sem(hi2c->Init.semlock);
    return ercd;
}
